""" Init file for SRPMath

Context : SRP.FITS
Module  : Math
Version : 1.0.0
Author  : Stefano Covino
Date    : 14/03/2017
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (14/03/2017) First version.
"""


__all__ = ['TriangleMatch',]


