""" Init file for SRPPolarimetry

Context : SRP
Module  : Polarimetry
Version : 1.6.0
Author  : Stefano Covino
Date    : 24/08/2017
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino

Usage   : to be imported

History : (15/02/2012) First named version.
        : (10/11/2015) resid, centerGauss and getBackground added.
        : (13/11/2015) MinMax added.
        : (18/11/2015) centerMoment added.
        : (19/04/2017) ApErr added.
        : (22/08/2017) DaoPhot added.
        : (23/08/2017) ApyPhot added.
        : (24/08/2017) centerObj added.
"""


__all__ = ['ApErr', 'ApyPhot', 'centerGauss', 'centerMoment', 'centerObj', 'Counts2Mag',
            'DaoPhot', 'Mag2Counts', 'getBackground', 'MinMax', 'resid']



