""" Utility functions and classes for SRP

Context : SRP
Module  : Frames
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/09/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (29/09/2010) First version.


"""


# Eclipse package
SRPpeak                 = "peak"
EclipseNotFound         =   -1
EclipseNotExecutable    =   -2