""" Init file for SRPFrames

Context : SRP
Module  : Frames
Version : 1.1.0
Author  : Stefano Covino
Date    : 16/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (31/08/2012) First named version.
        : (16/05/2017) DAOObjectClass added.
"""



__all__ = ['AstrometryClass', 'DAOObjectClass', 'EclipseConstants', 'EclipseObjectClass',
           'getCenterRADEC', 'Pixel2WCS', 'SexConstants', 'SexObjectClass',
           'SExtractorConstants', 'SourceObjectsClass', 'WCS2Pixel']


