""" Utility functions and classes for SRP

Context : SRP
Module  : Frames
Version : 1.0.0
Author  : Stefano Covino
Date    : 27/10/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/10/2014) First version.


"""


# Sex constants
SexFrameNotFound         =   -1
